@@ -29,9 +29,12 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Cambridge, MA 02110-1301, USA
 #include <QLabel>
 #include <QPainter>
 #include <QPainterPath>
+#include <QParallelAnimationGroup>
+#include <QPointer>
+#include <QPropertyAnimation>
+#include <QSequentialAnimationGroup>
 #include <QStylePainter>
 #include <QToolButton>
-#include <QToolTip>
 
 // KDE
 #include <kdebug.h>
@@ -48,6 +51,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Cambridge, MA 02110-1301, USA
 #include "paintutils.h"
 #include "thumbnailview.h"
 #include "timeutils.h"
+#include "tooltipwidget.h"
 #ifndef GWENVIEW_SEMANTICINFO_BACKEND_NONE
 #include "../semanticinfo/semanticinfodirmodel.h"
 #endif
@@ -193,9 +197,11 @@ struct PreviewItemDelegatePrivate {
 	QModelIndex mIndexUnderCursor;
 	int mThumbnailSize;
 	PreviewItemDelegate::ThumbnailDetails mDetails;
+	PreviewItemDelegate::ContextBarMode mContextBarMode;
+	Qt::TextElideMode mTextElideMode;
 
-	QLabel* mTipLabel;
-
+	QPointer<ToolTipWidget> mToolTip;
+	QScopedPointer<QAbstractAnimation> mToolTipAnimation;
 
 	QToolButton* createContextBarButton(const char* iconName) {
 		const int size = KIconLoader::global()->currentSize(KIconLoader::Small);
@@ -222,20 +228,25 @@ struct PreviewItemDelegatePrivate {
 
 
 	void showContextBar(const QRect& rect, const QPixmap& thumbnailPix) {
+		if (mContextBarMode == PreviewItemDelegate::NoContextBar) {
+			return;
+		}
 		mContextBar->adjustSize();
-		const int posX = (rect.width() - mContextBar->width()) / 2;
+		// Center bar in FullContextBar mode, left align in
+		// SelectionOnlyContextBar mode
+		const int posX = mContextBarMode == PreviewItemDelegate::FullContextBar
+			? (rect.width() - mContextBar->width()) / 2
+			: 0;
 		const int posY = qMax(CONTEXTBAR_MARGIN, mThumbnailSize - thumbnailPix.height() - mContextBar->height());
 		mContextBar->move(rect.topLeft() + QPoint(posX, posY));
 		mContextBar->show();
 	}
 
 
-	void initTipLabel() {
-		mTipLabel = new QLabel(mView);
-		mTipLabel->setAutoFillBackground(true);
-		mTipLabel->setFrameStyle(QFrame::Box | QFrame::Plain);
-		mTipLabel->setPalette(QToolTip::palette());
-		mTipLabel->hide();
+	void initToolTip() {
+		mToolTip = new ToolTipWidget(mView->viewport());
+		mToolTip->setOpacity(0);
+		mToolTip->show();
 	}
 
 
@@ -280,7 +291,7 @@ struct PreviewItemDelegatePrivate {
 		} else {
 			mContextBar->hide();
 			mSaveButton->hide();
-			mTipLabel->hide();
+			hideToolTip();
 		}
 	}
 
@@ -317,10 +328,9 @@ struct PreviewItemDelegatePrivate {
 
 
 	QPoint saveButtonPosition(const QRect& itemRect) const {
-		QSize frameSize = mSaveButton->sizeHint();
-		int textHeight = mView->fontMetrics().height();
-		int posX = itemRect.right() - CONTEXTBAR_MARGIN - frameSize.width();
-		int posY = itemRect.bottom() - CONTEXTBAR_MARGIN - textHeight - frameSize.height();
+		QSize buttonSize = mSaveButton->sizeHint();
+		int posX = itemRect.right() - buttonSize.width();
+		int posY = itemRect.top() + mThumbnailSize + 2 * ITEM_MARGIN - buttonSize.height();
 
 		return QPoint(posX, posY);
 	}
@@ -397,7 +407,7 @@ struct PreviewItemDelegatePrivate {
 		QString text;
 		QHash<QString, QString>::const_iterator it = mElidedTextCache.constFind(fullText);
 		if (it == mElidedTextCache.constEnd()) {
-			text = fm.elidedText(fullText, Qt::ElideRight, rect.width());
+			text = fm.elidedText(fullText, mTextElideMode, rect.width());
 			mElidedTextCache[fullText] = text;
 		} else {
 			text = it.value();
@@ -456,31 +466,97 @@ struct PreviewItemDelegatePrivate {
 			elided |= isTextElided(text);
 			textList << text;
 		}
+
+		// FIXME: Duplicated from drawText
+		const KFileItem fileItem = fileItemForIndex(index);
+		const bool isDirOrArchive = ArchiveUtils::fileItemIsDirOrArchive(fileItem);
 		if (mDetails & PreviewItemDelegate::DateDetail) {
-			// FIXME: Duplicated from drawText
-			const KFileItem fileItem = fileItemForIndex(index);
-			const KDateTime dt = TimeUtils::dateTimeForFileItem(fileItem);
-			const QString text = KGlobal::locale()->formatDateTime(dt);
-			elided |= isTextElided(text);
-			textList << text;
+			if (!ArchiveUtils::fileItemIsDirOrArchive(fileItem)) {
+				const KDateTime dt = TimeUtils::dateTimeForFileItem(fileItem);
+				const QString text = KGlobal::locale()->formatDateTime(dt);
+				elided |= isTextElided(text);
+				textList << text;
+			}
 		}
+
+		if (!isDirOrArchive && (mDetails & PreviewItemDelegate::ImageSizeDetail)) {
+			QSize fullSize;
+			QPixmap thumbnailPix = mView->thumbnailForIndex(index, &fullSize);
+			if (fullSize.isValid()) {
+				const QString text = QString("%1x%2").arg(fullSize.width()).arg(fullSize.height());
+				elided |= isTextElided(text);
+				textList << text;
+			}
+		}
+
+		if (!isDirOrArchive && (mDetails & PreviewItemDelegate::FileSizeDetail)) {
+			const KIO::filesize_t size = fileItem.size();
+			if (size > 0) {
+				const QString text = KIO::convertSize(size);
+				elided |= isTextElided(text);
+				textList << text;
+			}
+		}
+
 		if (!elided) {
-			mTipLabel->hide();
+			hideToolTip();
 			return;
 		}
-		mTipLabel->setText(textList.join("\n"));
+
+		bool newTipLabel = !mToolTip;
+		if (!mToolTip) {
+			initToolTip();
+		}
+		mToolTip->setText(textList.join("\n"));
+		QSize tipSize = mToolTip->sizeHint();
 
 		// Compute tip position
 		QRect rect = mView->visualRect(index);
-		const int textX = ITEM_MARGIN;
 		const int textY = ITEM_MARGIN + mThumbnailSize + ITEM_MARGIN;
-		const int margin = mTipLabel->frameWidth();
-		const QPoint tipPosition = rect.topLeft() + QPoint(textX - margin, textY - margin);
+		const int spacing = 1;
+		QRect geometry(
+			QPoint(rect.topLeft() + QPoint((rect.width() - tipSize.width()) / 2, textY + spacing)),
+			tipSize
+			);
+		if (geometry.left() < 0) {
+			geometry.moveLeft(0);
+		} else if (geometry.right() > mView->viewport()->width()) {
+			geometry.moveRight(mView->viewport()->width());
+		}
 
 		// Show tip
-		mTipLabel->adjustSize();
-		mTipLabel->move(tipPosition);
-		mTipLabel->show();
+		QParallelAnimationGroup* anim = new QParallelAnimationGroup();
+		QPropertyAnimation* fadeIn = new QPropertyAnimation(mToolTip, "opacity");
+		fadeIn->setStartValue(mToolTip->opacity());
+		fadeIn->setEndValue(1.);
+		anim->addAnimation(fadeIn);
+
+		if (newTipLabel) {
+			mToolTip->setGeometry(geometry);
+		} else {
+			QPropertyAnimation* move = new QPropertyAnimation(mToolTip, "geometry");
+			move->setStartValue(mToolTip->geometry());
+			move->setEndValue(geometry);
+			anim->addAnimation(move);
+		}
+
+		mToolTipAnimation.reset(anim);
+		mToolTipAnimation->start();
+	}
+
+	void hideToolTip() {
+		if (!mToolTip) {
+			return;
+		}
+		QSequentialAnimationGroup* anim = new QSequentialAnimationGroup();
+		anim->addPause(500);
+		QPropertyAnimation* fadeOut = new QPropertyAnimation(mToolTip, "opacity");
+		fadeOut->setStartValue(mToolTip->opacity());
+		fadeOut->setEndValue(0.);
+		anim->addAnimation(fadeOut);
+		mToolTipAnimation.reset(anim);
+		mToolTipAnimation->start();
+		QObject::connect(anim, SIGNAL(finished()), mToolTip, SLOT(deleteLater()));
 	}
 
 	int itemWidth() const {
@@ -503,6 +579,12 @@ struct PreviewItemDelegatePrivate {
 		if (mDetails & PreviewItemDelegate::DateDetail) {
 			textHeight += lineHeight;
 		}
+		if (mDetails & PreviewItemDelegate::ImageSizeDetail) {
+			textHeight += lineHeight;
+		}
+		if (mDetails & PreviewItemDelegate::FileSizeDetail) {
+			textHeight += lineHeight;
+		}
 		if (mDetails & PreviewItemDelegate::RatingDetail) {
 			textHeight += ratingRowHeight();
 		}
@@ -532,6 +614,24 @@ struct PreviewItemDelegatePrivate {
 		mRotateLeftButton->setEnabled(isImage);
 		mRotateRightButton->setEnabled(isImage);
 	}
+
+	void updateContextBar() {
+		if (mContextBarMode == PreviewItemDelegate::NoContextBar) {
+			mContextBar->hide();
+			return;
+		}
+		const int width = itemWidth();
+		const int buttonWidth = mRotateRightButton->sizeHint().width();
+		bool full = mContextBarMode == PreviewItemDelegate::FullContextBar;
+		mFullScreenButton->setVisible(full);
+		mRotateLeftButton->setVisible(full && width >= 3 * buttonWidth);
+		mRotateRightButton->setVisible(full && width >= 4 * buttonWidth);
+		mContextBar->adjustSize();
+	}
+
+	void updateViewGridSize() {
+		mView->setGridSize(QSize(itemWidth(), itemHeight()));
+	}
 };
 
 
@@ -543,6 +643,8 @@ PreviewItemDelegate::PreviewItemDelegate(ThumbnailView* view)
 	view->viewport()->installEventFilter(this);
 	d->mThumbnailSize = view->thumbnailSize();
 	d->mDetails = FileNameDetail;
+	d->mContextBarMode = FullContextBar;
+	d->mTextElideMode = Qt::ElideRight;
 
 	connect(view, SIGNAL(rowsRemovedSignal(const QModelIndex&, int, int)),
 		SLOT(slotRowsChanged()));
@@ -593,8 +695,6 @@ PreviewItemDelegate::PreviewItemDelegate(ThumbnailView* view)
 	d->mSaveButton->hide();
 	connect(d->mSaveButton, SIGNAL(clicked()),
 		SLOT(slotSaveClicked()) );
-
-	d->initTipLabel();
 }
 
 
@@ -604,7 +704,7 @@ PreviewItemDelegate::~PreviewItemDelegate() {
 
 
 QSize PreviewItemDelegate::sizeHint( const QStyleOptionViewItem & /*option*/, const QModelIndex & /*index*/ ) const {
-	return QSize( d->itemWidth(), d->itemHeight() );
+	return d->mView->gridSize();
 }
 
 
@@ -629,10 +729,11 @@ bool PreviewItemDelegate::eventFilter(QObject*, QEvent* event) {
 
 void PreviewItemDelegate::paint( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const {
 	int thumbnailSize = d->mThumbnailSize;
-	QPixmap thumbnailPix = d->mView->thumbnailForIndex(index);
+	QSize fullSize;
+	QPixmap thumbnailPix = d->mView->thumbnailForIndex(index, &fullSize);
 	const KFileItem fileItem = fileItemForIndex(index);
 	const bool opaque = !thumbnailPix.hasAlphaChannel();
-	const bool isDirOrArchive = ArchiveUtils::fileItemIsDirOrArchive(fileItemForIndex(index));
+	const bool isDirOrArchive = ArchiveUtils::fileItemIsDirOrArchive(fileItem);
 	QRect rect = option.rect;
 	const bool selected = option.state & QStyle::State_Selected;
 	const bool underMouse = option.state & QStyle::State_MouseOver;
@@ -704,6 +805,15 @@ void PreviewItemDelegate::paint( QPainter * painter, const QStyleOptionViewItem
 		painter->drawPixmap(framePosition, d->mSaveButtonPixmap);
 	}
 
+	// Draw busy indicator
+	if (d->mView->isBusy(index)) {
+		QPixmap pix = d->mView->busySequenceCurrentPixmap();
+		painter->drawPixmap(
+			thumbnailRect.left() + (thumbnailRect.width() - pix.width()) / 2,
+			thumbnailRect.top() + (thumbnailRect.height() - pix.height()) / 2,
+			pix);
+	}
+
 	if (index == d->mIndexUnderCursor) {
 		// Show bar again: if the thumbnail has changed, we may need to update
 		// its position. Don't do it if we are over rotate buttons, though: it
@@ -735,6 +845,24 @@ void PreviewItemDelegate::paint( QPainter * painter, const QStyleOptionViewItem
 	if (!isDirOrArchive && (d->mDetails & PreviewItemDelegate::DateDetail)) {
 		const KDateTime dt = TimeUtils::dateTimeForFileItem(fileItem);
 		d->drawText(painter, textRect, fgColor, KGlobal::locale()->formatDateTime(dt));
+		textRect.moveTop(textRect.bottom());
+	}
+
+	if (!isDirOrArchive && (d->mDetails & PreviewItemDelegate::ImageSizeDetail)) {
+		if (fullSize.isValid()) {
+			const QString text = QString("%1x%2").arg(fullSize.width()).arg(fullSize.height());
+			d->drawText(painter, textRect, fgColor, text);
+			textRect.moveTop(textRect.bottom());
+		}
+	}
+
+	if (!isDirOrArchive && (d->mDetails & PreviewItemDelegate::FileSizeDetail)) {
+		const KIO::filesize_t size = fileItem.size();
+		if (size > 0) {
+			const QString st = KIO::convertSize(size);
+			d->drawText(painter, textRect, fgColor, st);
+			textRect.moveTop(textRect.bottom());
+		}
 	}
 
 	if (!isDirOrArchive && (d->mDetails & PreviewItemDelegate::RatingDetail)) {
@@ -747,13 +875,8 @@ void PreviewItemDelegate::paint( QPainter * painter, const QStyleOptionViewItem
 
 void PreviewItemDelegate::setThumbnailSize(int value) {
 	d->mThumbnailSize = value;
-
-	const int width = d->itemWidth();
-	const int buttonWidth = d->mRotateRightButton->sizeHint().width();
-	d->mRotateLeftButton->setVisible(width >= 3 * buttonWidth);
-	d->mRotateRightButton->setVisible(width >= 4 * buttonWidth);
-	d->mContextBar->adjustSize();
-
+	d->updateViewGridSize();
+	d->updateContextBar();
 	d->mElidedTextCache.clear();
 }
 
@@ -793,10 +916,37 @@ PreviewItemDelegate::ThumbnailDetails PreviewItemDelegate::thumbnailDetails() co
 
 void PreviewItemDelegate::setThumbnailDetails(PreviewItemDelegate::ThumbnailDetails details) {
 	d->mDetails = details;
+	d->updateViewGridSize();
 	d->mView->scheduleDelayedItemsLayout();
 }
 
 
+PreviewItemDelegate::ContextBarMode PreviewItemDelegate::contextBarMode() const {
+	return d->mContextBarMode;
+}
+
+
+void PreviewItemDelegate::setContextBarMode(PreviewItemDelegate::ContextBarMode mode) {
+	d->mContextBarMode = mode;
+	d->updateContextBar();
+}
+
+
+Qt::TextElideMode PreviewItemDelegate::textElideMode() const {
+	return d->mTextElideMode;
+}
+
+
+void PreviewItemDelegate::setTextElideMode(Qt::TextElideMode mode) {
+	if (d->mTextElideMode == mode) {
+		return;
+	}
+	d->mTextElideMode = mode;
+	d->mElidedTextCache.clear();
+	d->mView->viewport()->update();
+}
+
+
 void PreviewItemDelegate::slotRowsChanged() {
 	// We need to update hover ui because the current index may have
 	// disappeared: for example if the current image is removed with "del".
