@@ -29,14 +29,21 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <QPainter>
 #include <QPointer>
 #include <QQueue>
+#include <QScrollBar>
+#include <QTimeLine>
 #include <QTimer>
+#include <QToolTip>
 
 // KDE
 #include <kdebug.h>
 #include <kdirmodel.h>
+#include <kicon.h>
+#include <kiconloader.h>
 #include <kglobalsettings.h>
+#include <kpixmapsequence.h>
 
 // Local
+#include "abstractdocumentinfoprovider.h"
 #include "abstractthumbnailviewhelper.h"
 #include "archiveutils.h"
 #include "mimetypeutils.h"
@@ -53,12 +60,12 @@ namespace Gwenview {
 #define LOG(x) ;
 #endif
 
-/** How many pixels between items */
-const int SPACING = 11;
-
 /** How many msec to wait before starting to smooth thumbnails */
 const int SMOOTH_DELAY = 500;
 
+const int DRAG_THUMB_SIZE = KIconLoader::SizeHuge;
+const int DRAG_THUMB_SPACING = 4;
+
 static KFileItem fileItemForIndex(const QModelIndex& index) {
 	if (!index.isValid()) {
 		LOG("Invalid index");
@@ -77,19 +84,26 @@ static KUrl urlForIndex(const QModelIndex& index) {
 struct Thumbnail {
 	Thumbnail(const QPersistentModelIndex& index_)
 	: mIndex(index_)
-	, mRough(true) {}
+	, mRough(true)
+	, mWaitingForThumbnail(true) {}
 
-	Thumbnail() : mRough(true) {}
+	Thumbnail()
+	: mRough(true)
+	, mWaitingForThumbnail(true) {}
 
 	/**
 	 * Init the thumbnail based on a icon
 	 */
 	void initAsIcon(const QPixmap& pix) {
 		mGroupPix = pix;
-		mFullSize = pix.size();
+		int largeGroupSize = ThumbnailGroup::pixelSize(ThumbnailGroup::Large);
+		mFullSize = QSize(largeGroupSize, largeGroupSize);
 	}
 
 	bool isGroupPixAdaptedForSize(int size) const {
+		if (mWaitingForThumbnail) {
+			return false;
+		}
 		if (mGroupPix.isNull()) {
 			return false;
 		}
@@ -110,17 +124,24 @@ struct Thumbnail {
 	QPixmap mAdjustedPix;
 	/// Size of the full image
 	QSize mFullSize;
+	/// Real size of the full image, invalid unless the thumbnail
+	/// represents a raster image (not an icon)
+	QSize mRealFullSize;
 	/// Whether mAdjustedPix represents has been scaled using fast or smooth
 	//transformation
 	bool mRough;
+	/// Set to true if mGroupPix should be replaced with a real thumbnail
+	bool mWaitingForThumbnail;
 };
 
 typedef QHash<QUrl, Thumbnail> ThumbnailForUrl;
 typedef QQueue<KUrl> UrlQueue;
+typedef QSet<QPersistentModelIndex> PersistentModelIndexSet;
 
 struct ThumbnailViewPrivate {
 	ThumbnailView* that;
 	int mThumbnailSize;
+	AbstractDocumentInfoProvider* mDocumentInfoProvider;
 	AbstractThumbnailViewHelper* mThumbnailViewHelper;
 	ThumbnailForUrl mThumbnailForUrl;
 	QTimer mScheduledThumbnailGenerationTimer;
@@ -131,6 +152,20 @@ struct ThumbnailViewPrivate {
 	QPixmap mWaitingThumbnail;
 	QPointer<ThumbnailLoadJob> mThumbnailLoadJob;
 
+	PersistentModelIndexSet mBusyIndexSet;
+	KPixmapSequence mBusySequence;
+	QTimeLine* mBusyAnimationTimeLine;
+
+	void setupBusyAnimation() {
+		mBusySequence = KPixmapSequence("process-working", 22);
+		mBusyAnimationTimeLine = new QTimeLine(100 * mBusySequence.frameCount(), that);
+		mBusyAnimationTimeLine->setCurveShape(QTimeLine::LinearCurve);
+		mBusyAnimationTimeLine->setEndFrame(mBusySequence.frameCount() - 1);
+		mBusyAnimationTimeLine->setLoopCount(0);
+		QObject::connect(mBusyAnimationTimeLine, SIGNAL(frameChanged(int)),
+			that, SLOT(updateBusyIndexes()));
+	}
+
 	void scheduleThumbnailGenerationForVisibleItems() {
 		if (mThumbnailLoadJob) {
 			mThumbnailLoadJob->removeItems(mThumbnailLoadJob->pendingItems());
@@ -140,12 +175,13 @@ struct ThumbnailViewPrivate {
 	}
 
 	void updateThumbnailForModifiedDocument(const QModelIndex& index) {
+		Q_ASSERT(mDocumentInfoProvider);
 		KFileItem item = fileItemForIndex(index);
 		KUrl url = item.url();
 		ThumbnailGroup::Enum group = ThumbnailGroup::fromPixelSize(mThumbnailSize);
 		QPixmap pix;
 		QSize fullSize;
-		mThumbnailViewHelper->thumbnailForDocument(url, group, &pix, &fullSize);
+		mDocumentInfoProvider->thumbnailForDocument(url, group, &pix, &fullSize);
 		mThumbnailForUrl[url] = Thumbnail(QPersistentModelIndex(index));
 		that->setThumbnail(item, pix, fullSize);
 	}
@@ -179,6 +215,68 @@ struct ThumbnailViewPrivate {
 			thumbnail->mRough = true;
 		}
 	}
+
+	QPixmap dragPixmapForIndex(const QModelIndex& index) const {
+		KUrl url = urlForIndex(index);
+		QPixmap pix = mThumbnailForUrl.value(url).mAdjustedPix;
+		if (qMax(pix.width(), pix.height()) > DRAG_THUMB_SIZE) {
+			return pix.scaled(DRAG_THUMB_SIZE, DRAG_THUMB_SIZE, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+		} else {
+			return pix;
+		}
+	}
+
+	QPixmap createDragPixmap(const QModelIndexList& indexes) {
+		const int MAX_THUMBS = 3;
+
+		int thumbCount;
+		bool more;
+		if (indexes.count() > MAX_THUMBS) {
+			thumbCount = MAX_THUMBS;
+			more = true;
+		} else {
+			thumbCount = indexes.count();
+			more = false;
+		}
+		QList<QPixmap> thumbs;
+		int width = 0;
+		int height = 0;
+		for (int row=0; row<thumbCount; ++row) {
+			QModelIndex index;
+			if (row == thumbCount-1 && more) {
+				QString text = "(...)";
+				QPixmap pix(that->fontMetrics().boundingRect(text).size());
+				pix.fill(Qt::transparent);
+				{
+					QPainter painter(&pix);
+					painter.drawText(pix.rect(), Qt::AlignHCenter | Qt::AlignBottom, text);
+				}
+				index = indexes.last();
+				width += pix.width();
+				thumbs << pix;
+			} else {
+				index = indexes[row];
+			}
+			QPixmap thumb = dragPixmapForIndex(index);
+			height = qMax(height, thumb.height());
+			width += thumb.width();
+			thumbs << thumb;
+		}
+
+		QPixmap pix(
+			width + (thumbs.count() + 1) * DRAG_THUMB_SPACING,
+			height + 2 * DRAG_THUMB_SPACING
+			);
+		pix.fill(QToolTip::palette().color(QPalette::Inactive, QPalette::ToolTipBase));
+		QPainter painter(&pix);
+
+		int x = DRAG_THUMB_SPACING;
+		Q_FOREACH(const QPixmap& thumb, thumbs) {
+			painter.drawPixmap(x, (pix.height() - thumb.height()) / 2, thumb);
+			x += thumb.width() + DRAG_THUMB_SPACING;
+		}
+		return pix;
+	}
 };
 
 
@@ -187,14 +285,21 @@ ThumbnailView::ThumbnailView(QWidget* parent)
 , d(new ThumbnailViewPrivate) {
 	d->that = this;
 	d->mThumbnailViewHelper = 0;
+	d->mDocumentInfoProvider = 0;
+	// Init to some stupid value so that the first call to setThumbnailSize()
+	// is not ignored (do not use 0 in case someone try to divide by
+	// mThumbnailSize...)
+	d->mThumbnailSize = 1;
 
 	setFrameShape(QFrame::NoFrame);
 	setViewMode(QListView::IconMode);
 	setResizeMode(QListView::Adjust);
-	setSpacing(SPACING);
 	setDragEnabled(true);
 	setAcceptDrops(true);
 	setDropIndicatorShown(true);
+	setUniformItemSizes(true);
+
+	d->setupBusyAnimation();
 
 	viewport()->setMouseTracking(true);
 	// Set this attribute, otherwise the item delegate won't get the
@@ -204,11 +309,6 @@ ThumbnailView::ThumbnailView(QWidget* parent)
 	setVerticalScrollMode(ScrollPerPixel);
 	setHorizontalScrollMode(ScrollPerPixel);
 
-	// Make sure mThumbnailSize is initialized before calling setThumbnailSize,
-	// since it will compare the new size with the old one
-	d->mThumbnailSize = 0;
-	setThumbnailSize(128);
-
 	d->mScheduledThumbnailGenerationTimer.setSingleShot(true);
 	d->mScheduledThumbnailGenerationTimer.setInterval(500);
 	connect(&d->mScheduledThumbnailGenerationTimer, SIGNAL(timeout()),
@@ -284,7 +384,6 @@ void ThumbnailView::setThumbnailSize(int value) {
 	}
 
 	thumbnailSizeChanged(value);
-	setSpacing(SPACING);
 	d->scheduleThumbnailGenerationForVisibleItems();
 }
 
@@ -303,6 +402,21 @@ AbstractThumbnailViewHelper* ThumbnailView::thumbnailViewHelper() const {
 }
 
 
+void ThumbnailView::setDocumentInfoProvider(AbstractDocumentInfoProvider* provider) {
+	d->mDocumentInfoProvider = provider;
+	if (provider) {
+		connect(provider, SIGNAL(busyStateChanged(const QModelIndex&, bool)),
+			SLOT(updateThumbnailBusyState(const QModelIndex&, bool)));
+		connect(provider, SIGNAL(documentChanged(const QModelIndex&)),
+			SLOT(updateThumbnail(const QModelIndex&)));
+	}
+}
+
+AbstractDocumentInfoProvider* ThumbnailView::documentInfoProvider() const {
+	return d->mDocumentInfoProvider;
+}
+
+
 void ThumbnailView::rowsAboutToBeRemoved(const QModelIndex& parent, int start, int end) {
 	QListView::rowsAboutToBeRemoved(parent, start, end);
 
@@ -352,6 +466,12 @@ void ThumbnailView::rowsInserted(const QModelIndex& parent, int start, int end)
 }
 
 
+void ThumbnailView::dataChanged(const QModelIndex& topLeft, const QModelIndex& bottomRight) {
+	QListView::dataChanged(topLeft, bottomRight);
+	d->mScheduledThumbnailGenerationTimer.start();
+}
+
+
 void ThumbnailView::showContextMenu() {
 	d->mThumbnailViewHelper->showContextMenu(this);
 }
@@ -372,7 +492,10 @@ void ThumbnailView::setThumbnail(const KFileItem& item, const QPixmap& pixmap, c
 	Thumbnail& thumbnail = it.value();
 	thumbnail.mGroupPix = pixmap;
 	thumbnail.mAdjustedPix = QPixmap();
-	thumbnail.mFullSize = size;
+	int largeGroupSize = ThumbnailGroup::pixelSize(ThumbnailGroup::Large);
+	thumbnail.mFullSize = size.isValid() ? size : QSize(largeGroupSize, largeGroupSize);
+	thumbnail.mRealFullSize = size;
+	thumbnail.mWaitingForThumbnail = false;
 
 	update(thumbnail.mIndex);
 }
@@ -384,39 +507,69 @@ void ThumbnailView::setBrokenThumbnail(const KFileItem& item) {
 		return;
 	}
 	Thumbnail& thumbnail = it.value();
-	if (MimeTypeUtils::fileItemKind(item) == MimeTypeUtils::KIND_VIDEO) {
+	MimeTypeUtils::Kind kind = MimeTypeUtils::fileItemKind(item);
+	if (kind == MimeTypeUtils::KIND_VIDEO) {
 		// Special case for videos because our kde install may come without
 		// support for video thumbnails so we show the mimetype icon instead of
 		// a broken image icon
-		thumbnail.initAsIcon(item.pixmap(128));
+		ThumbnailGroup::Enum group = ThumbnailGroup::fromPixelSize(d->mThumbnailSize);
+		QPixmap pix = item.pixmap(ThumbnailGroup::pixelSize(group));
+		thumbnail.initAsIcon(pix);
+	} else if (kind == MimeTypeUtils::KIND_DIR) {
+		// Special case for folders because ThumbnailLoadJob does not return a
+		// thumbnail if there is no images
+		thumbnail.mWaitingForThumbnail = false;
+		return;
 	} else {
 		thumbnail.initAsIcon(DesktopIcon("image-missing", 48));
+		thumbnail.mFullSize = thumbnail.mGroupPix.size();
 	}
 	update(thumbnail.mIndex);
 }
 
 
-QPixmap ThumbnailView::thumbnailForIndex(const QModelIndex& index) {
+QPixmap ThumbnailView::thumbnailForIndex(const QModelIndex& index, QSize* fullSize) {
 	KFileItem item = fileItemForIndex(index);
 	if (item.isNull()) {
 		LOG("Invalid item");
+		if (fullSize) {
+			*fullSize = QSize();
+		}
 		return QPixmap();
 	}
 	KUrl url = item.url();
 
+	// Find or create Thumbnail instance
 	ThumbnailForUrl::Iterator it = d->mThumbnailForUrl.find(url);
 	if (it == d->mThumbnailForUrl.end()) {
-		if (ArchiveUtils::fileItemIsDirOrArchive(item)) {
-			Thumbnail thumbnail = Thumbnail(QPersistentModelIndex(index));
-			thumbnail.initAsIcon(item.pixmap(128));
-			it = d->mThumbnailForUrl.insert(url, thumbnail);
-		} else {
-			return d->mWaitingThumbnail;
+		Thumbnail thumbnail = Thumbnail(QPersistentModelIndex(index));
+		it = d->mThumbnailForUrl.insert(url, thumbnail);
+	}
+	Thumbnail& thumbnail = it.value();
+
+	// If dir or archive, generate a thumbnail from fileitem pixmap
+	MimeTypeUtils::Kind kind = MimeTypeUtils::fileItemKind(item);
+	if (kind == MimeTypeUtils::KIND_ARCHIVE || kind == MimeTypeUtils::KIND_DIR) {
+		int groupSize = ThumbnailGroup::pixelSize(ThumbnailGroup::fromPixelSize(d->mThumbnailSize));
+		if (thumbnail.mGroupPix.isNull() || thumbnail.mGroupPix.width() < groupSize) {
+			QPixmap pix = item.pixmap(groupSize);
+			thumbnail.initAsIcon(pix);
+			if (kind == MimeTypeUtils::KIND_ARCHIVE) {
+				// No thumbnails for archives
+				thumbnail.mWaitingForThumbnail = false;
+			} else {
+				// set mWaitingForThumbnail to true (necessary in the case
+				// 'thumbnail' already existed before, but with a too small
+				// mGroupPix)
+				thumbnail.mWaitingForThumbnail = true;
+			}
 		}
 	}
 
-	Thumbnail& thumbnail = it.value();
 	if (thumbnail.mGroupPix.isNull()) {
+		if (fullSize) {
+			*fullSize = QSize();
+		}
 		return d->mWaitingThumbnail;
 	}
 
@@ -430,13 +583,41 @@ QPixmap ThumbnailView::thumbnailForIndex(const QModelIndex& index) {
 			d->mSmoothThumbnailTimer.start(SMOOTH_DELAY);
 		}
 	}
+	if (fullSize) {
+		*fullSize = thumbnail.mRealFullSize;
+	}
 	return thumbnail.mAdjustedPix;
 }
 
 
 bool ThumbnailView::isModified(const QModelIndex& index) const {
+	if (!d->mDocumentInfoProvider) {
+		return false;
+	}
 	KUrl url = urlForIndex(index);
-	return d->mThumbnailViewHelper->isDocumentModified(url);
+	return d->mDocumentInfoProvider->isModified(url);
+}
+
+
+bool ThumbnailView::isBusy(const QModelIndex& index) const {
+	if (!d->mDocumentInfoProvider) {
+		return false;
+	}
+	KUrl url = urlForIndex(index);
+	return d->mDocumentInfoProvider->isBusy(url);
+}
+
+
+void ThumbnailView::startDrag(Qt::DropActions supportedActions) {
+	QModelIndexList indexes = selectionModel()->selectedIndexes();
+	if (indexes.isEmpty()) {
+		return;
+	}
+	QDrag* drag = new QDrag(this);
+	drag->setMimeData(model()->mimeData(indexes));
+	QPixmap pix = d->createDragPixmap(indexes);
+	drag->setPixmap(pix);
+	drag->exec(supportedActions, Qt::CopyAction);
 }
 
 
@@ -501,6 +682,17 @@ void ThumbnailView::showEvent(QShowEvent* event) {
 }
 
 
+void ThumbnailView::wheelEvent(QWheelEvent* event) {
+	// If we don't adjust the single step, the wheel scroll exactly one item up
+	// and down, giving the impression that the items do not move but only
+	// their label changes.
+	// For some reason it is necessary to set the step here: setting it in
+	// setThumbnailSize() does not work
+	//verticalScrollBar()->setSingleStep(d->mThumbnailSize / 5);
+	QListView::wheelEvent(event);
+}
+
+
 void ThumbnailView::scrollToSelectedIndex() {
 	QModelIndexList list = selectedIndexes();
 	if (list.count() >= 1) {
@@ -522,7 +714,7 @@ void ThumbnailView::scrollContentsBy(int dx, int dy) {
 
 
 void ThumbnailView::generateThumbnailsForVisibleItems() {
-	if (!isVisible()) {
+	if (!isVisible() || !model()) {
 		return;
 	}
 	KFileItemList list;
@@ -546,14 +738,14 @@ void ThumbnailView::generateThumbnailsForVisibleItems() {
 			continue;
 		}
 
-		// Filter out non documents
+		// Filter out archives
 		MimeTypeUtils::Kind kind = MimeTypeUtils::fileItemKind(item);
-		if (kind == MimeTypeUtils::KIND_DIR || kind == MimeTypeUtils::KIND_ARCHIVE) {
+		if (kind == MimeTypeUtils::KIND_ARCHIVE) {
 			continue;
 		}
 
 		// Immediately update modified items
-		if (d->mThumbnailViewHelper->isDocumentModified(url)) {
+		if (d->mDocumentInfoProvider && d->mDocumentInfoProvider->isModified(url)) {
 			d->updateThumbnailForModifiedDocument(index);
 			continue;
 		}
@@ -564,10 +756,15 @@ void ThumbnailView::generateThumbnailsForVisibleItems() {
 			continue;
 		}
 
-		// Add the item to our list and to mThumbnailForUrl, so that
-		// setThumbnail() can find the item to update
+		// Add the item to our list
 		list << item;
-		d->mThumbnailForUrl[url] = Thumbnail(QPersistentModelIndex(index));
+
+		// Insert the thumbnail in mThumbnailForUrl, so that
+		// setThumbnail() can find the item to update
+		if (it == d->mThumbnailForUrl.constEnd()) {
+			Thumbnail thumbnail = Thumbnail(QPersistentModelIndex(index));
+			d->mThumbnailForUrl.insert(url, thumbnail);
+		}
 	}
 
 	if (!list.empty()) {
@@ -576,10 +773,10 @@ void ThumbnailView::generateThumbnailsForVisibleItems() {
 }
 
 
-void ThumbnailView::generateThumbnailForIndex(const QModelIndex& index) {
+void ThumbnailView::updateThumbnail(const QModelIndex& index) {
 	KFileItem item = fileItemForIndex(index);
 	KUrl url = item.url();
-	if (d->mThumbnailViewHelper->isDocumentModified(url)) {
+	if (d->mDocumentInfoProvider && d->mDocumentInfoProvider->isModified(url)) {
 		d->updateThumbnailForModifiedDocument(index);
 	} else {
 		KFileItemList list;
@@ -589,6 +786,35 @@ void ThumbnailView::generateThumbnailForIndex(const QModelIndex& index) {
 }
 
 
+void ThumbnailView::updateThumbnailBusyState(const QModelIndex& _index, bool busy) {
+	QPersistentModelIndex index(_index);
+	if (busy && !d->mBusyIndexSet.contains(index)) {
+		d->mBusyIndexSet << index;
+		update(index);
+		if (d->mBusyAnimationTimeLine->state() != QTimeLine::Running) {
+			d->mBusyAnimationTimeLine->start();
+		}
+	} else if (!busy && d->mBusyIndexSet.remove(index)) {
+		update(index);
+		if (d->mBusyIndexSet.isEmpty()) {
+			d->mBusyAnimationTimeLine->stop();
+		}
+	}
+}
+
+
+void ThumbnailView::updateBusyIndexes() {
+	Q_FOREACH(const QPersistentModelIndex& index, d->mBusyIndexSet) {
+		update(index);
+	}
+}
+
+
+QPixmap ThumbnailView::busySequenceCurrentPixmap() const {
+	return d->mBusySequence.frameAt(d->mBusyAnimationTimeLine->currentFrame());
+}
+
+
 void ThumbnailView::smoothNextThumbnail() {
 	if (d->mSmoothThumbnailQueue.isEmpty()) {
 		return;
