@@ -6,8 +6,8 @@
  * Date        : 2006-12-09
  * Description : a tread-safe libraw C++ program interface
  *
- * Copyright (C) 2006-2009 by Gilles Caulier <caulier dot gilles at gmail dot com> 
- * Copyright (C) 2006-2009 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
+ * Copyright (C) 2006-2010 by Gilles Caulier <caulier dot gilles at gmail dot com> 
+ * Copyright (C) 2006-2010 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
  * Copyright (C) 2007-2008 by Guillaume Castagnino <casta at xwing dot info>
  *
  * This program is free software; you can redistribute it
@@ -191,7 +191,7 @@ bool KDcraw::loadHalfPreview(QImage& image, const QString& path)
         return false;
     }
 
-    libraw_processed_image_t *halfImg = raw.dcraw_make_mem_image(&ret);
+    libraw_processed_image_t* halfImg = raw.dcraw_make_mem_image(&ret);
     if(!halfImg)
     {
         kDebug(51002) << "LibRaw: failed to run dcraw_make_mem_image: " << libraw_strerror(ret) << endl;
@@ -250,6 +250,14 @@ bool KDcraw::rawFileIdentify(DcrawInfoContainer& identify, const QString& path)
 // ----------------------------------------------------------------------------------
 
 bool KDcraw::extractRAWData(const QString& filePath, QByteArray& rawData, DcrawInfoContainer& identify)
+{
+    return extractRAWData(filePath, rawData, identify, false, 0);
+}
+
+// ----------------------------------------------------------------------------------
+
+bool KDcraw::extractRAWData(const QString& filePath, QByteArray& rawData, DcrawInfoContainer& identify,
+                            bool addMaskedBorders, unsigned int shotSelect)
 {
     QFileInfo fileInfo(filePath);
     QString   rawFilesExt(rawFiles());
@@ -285,6 +293,7 @@ bool KDcraw::extractRAWData(const QString& filePath, QByteArray& rawData, DcrawI
 
     raw.imgdata.params.output_bps    = 16;
     raw.imgdata.params.document_mode = 2;
+    raw.imgdata.params.shot_select   = shotSelect;
 
     ret = raw.unpack();
     if (ret != LIBRAW_SUCCESS)
@@ -294,6 +303,18 @@ bool KDcraw::extractRAWData(const QString& filePath, QByteArray& rawData, DcrawI
         return false;
     }
 
+    if (addMaskedBorders)
+    {
+
+        ret = raw.add_masked_borders_to_bitmap();
+        if (ret != LIBRAW_SUCCESS)
+        {
+            kDebug(51002) << "LibRaw: failed to add masked borders: " << libraw_strerror(ret) << endl;
+            raw.recycle();
+            return false;
+        }
+    }
+
     if (m_cancel)
     {
         raw.recycle();
@@ -319,7 +340,7 @@ bool KDcraw::extractRAWData(const QString& filePath, QByteArray& rawData, DcrawI
     {
         for (uint col=0 ; col < raw.imgdata.sizes.iwidth ; col++)
         {
-            *output = raw.imgdata.image[raw.imgdata.sizes.iwidth*row + col] [raw.FC(row, col)];
+            *output = raw.imgdata.image[raw.imgdata.sizes.iwidth*row + col] [raw.COLOR(row, col)];
             *output++;
         }
     }
@@ -629,6 +650,7 @@ bool KDcraw::loadFromLibraw(const QString& filePath, QByteArray& imageData,
 
     args.append(filePath);
     kDebug(51002) << "LibRaw: dcraw emulation: " << args << endl;
+    kDebug(51002) << "LibRaw highlights adjustements : " << m_rawDecodingSettings.fixColorsHighlights << endl;
 
     int ret = raw.open_file(QFile::encodeName(filePath));
     if (ret != LIBRAW_SUCCESS)
@@ -660,6 +682,19 @@ bool KDcraw::loadFromLibraw(const QString& filePath, QByteArray& imageData,
     }
     d->setProgress(0.25);
 
+    if (m_rawDecodingSettings.fixColorsHighlights)
+    {
+        kDebug(51002) << "Applying LibRaw highlights adjustements" << endl;
+        // 1.0 is fallback to default value
+        raw.imgdata.params.adjust_maximum_thr = 1.0; 
+    }
+    else
+    {
+        kDebug(51002) << "Disabling LibRaw highlights adjustements" << endl;
+        // 0.0 disables this feature
+        raw.imgdata.params.adjust_maximum_thr = 0.0; 
+    }
+
     ret = raw.dcraw_process();
     if (ret != LIBRAW_SUCCESS)
     {
@@ -675,7 +710,7 @@ bool KDcraw::loadFromLibraw(const QString& filePath, QByteArray& imageData,
     }
     d->setProgress(0.3);
 
-    libraw_processed_image_t *img = raw.dcraw_make_mem_image(&ret);
+    libraw_processed_image_t* img = raw.dcraw_make_mem_image(&ret);
     if(!img)
     {
         kDebug(51002) << "LibRaw: failed to run dcraw_make_mem_image: " << libraw_strerror(ret) << endl;
@@ -709,7 +744,7 @@ bool KDcraw::loadFromLibraw(const QString& filePath, QByteArray& imageData,
     return true;
 }
 
-const char *KDcraw::rawFiles()
+const char* KDcraw::rawFiles()
 {
     return raw_file_extentions;
 }
