@@ -498,6 +498,12 @@ class OKULAR_EXPORT Document : public QObject
          * Prints the document to the given @p printer.
          */
         bool print( QPrinter &printer );
+        
+        /**
+         * Returns the last print error in case print() failed
+         * @since 0.11 (KDE 4.5)
+         */
+        QString printError() const;
 
         /**
          * Returns a custom printer configuration page or 0 if no
@@ -542,6 +548,14 @@ class OKULAR_EXPORT Document : public QObject
          */
         bool saveChanges( const QString &fileName );
 
+        /**
+         * Save the document and the optional changes to it to the specified
+         * @p fileName and returns a @p errorText if fails.
+         *
+         * @since 0.10 (KDE 4.4)
+         */
+        bool saveChanges( const QString &fileName, QString *errorText );
+
         /**
          * Register the specified @p view for the current document.
          *
@@ -579,6 +593,18 @@ class OKULAR_EXPORT Document : public QObject
          */
         bool saveDocumentArchive( const QString &fileName );
 
+        /**
+         * Asks the generator to dynamically generate a SourceReference for a given
+         * page number and absolute X and Y position on this page.
+         *
+         * @attention Ownership of the returned SourceReference is transferred to the caller.
+         * @note This method does not call processSourceReference( const SourceReference * )
+         *
+         * @since 0.10 (KDE 4.4)
+        */
+        const SourceReference * dynamicSourceReference( int pageNr, double absX, double absY );
+
+
     public Q_SLOTS:
         /**
          * This slot is called whenever the user changes the @p rotation of
@@ -809,7 +835,10 @@ class OKULAR_EXPORT DocumentInfo : public QDomDocument
             ModificationDate,   ///< The date of last modification of the document
             MimeType,           ///< The mime type of the document
             Category,           ///< The category of the document
-            Keywords            ///< The keywords which describe the content of the document
+            Keywords,           ///< The keywords which describe the content of the document
+            FilePath,           ///< The path of the file @since 0.10 (KDE 4.4)
+            DocumentSize,       ///< The size of the document @since 0.10 (KDE 4.4)
+            PagesSize           ///< The size of the pages (if all pages have the same size) @since 0.10 (KDE 4.4)
         };
 
         /**
@@ -828,13 +857,26 @@ class OKULAR_EXPORT DocumentInfo : public QDomDocument
          * Sets the value for a predefined key. You should use this method
          * whenever a predefined key exists for your value.
          */
-        void set( enum Key, const QString &value );
+        void set( Key key, const QString &value );
 
         /**
          * Returns the value for a given key or an empty string when the
          * key doesn't exist.
          */
         QString get( const QString &key ) const;
+
+        /**
+         * Returns the internal string for the given key
+         * @since 0.10 (KDE 4.4)
+         */
+        static QString getKeyString( Key key );
+
+        /**
+         * Returns the user visible string for the given key
+         * @since 0.10 (KDE 4.4)
+         */
+        static QString getKeyTitle( Key key );
+
 };
 
 /**
@@ -961,3 +1003,5 @@ Q_DECLARE_METATYPE( Okular::DocumentInfo::Key )
 Q_DECLARE_OPERATORS_FOR_FLAGS( Okular::Document::PixmapRequestFlags )
 
 #endif
+
+/* kate: replace-tabs on; indent-width 4; */
