@@ -25,7 +25,7 @@ static KAboutData createAboutData()
          "okular_comicbook",
          "okular_comicbook",
          ki18n( "ComicBook Backend" ),
-         "0.1.3",
+         "0.2",
          ki18n( "A renderer for various comic book formats" ),
          KAboutData::License_GPL,
          ki18n( "© 2007-2008 Tobias Koenig" )
@@ -95,6 +95,7 @@ bool ComicBookGenerator::print( QPrinter& printer )
     QPainter p( &printer );
 
     QList<int> pageList = Okular::FilePrinter::pageList( printer, document()->pages(),
+                                                         document()->currentPage() + 1,
                                                          document()->bookmarkedPageList() );
 
     for ( int i = 0; i < pageList.count(); ++i ) {
