@@ -1,6 +1,6 @@
 /***************************************************************************
  *   Copyright (C) 2004 by Enrico Ros <eros.kde@email.it>                  *
- *   Copyright (C) 2007 by Pino Toscano <pino@kde.org>                     *
+ *   Copyright (C) 2007, 2009-2010 by Pino Toscano <pino@kde.org>          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -15,14 +15,18 @@
 
 // qt/kde includes
 #include <qapplication.h>
+#include <qlayout.h>
 #include <qtimer.h>
 #include <kcolorscheme.h>
+#include <kpixmapsequence.h>
+#include <kpixmapsequencewidget.h>
 
 SearchLineEdit::SearchLineEdit( QWidget * parent, Okular::Document * document )
     : KLineEdit( parent ), m_document( document ), m_minLength( 0 ),
       m_caseSensitivity( Qt::CaseInsensitive ),
       m_searchType( Okular::Document::AllDocument ), m_id( -1 ),
-      m_moveViewport( false ), m_changed( false ), m_fromStart( true )
+      m_moveViewport( false ), m_changed( false ), m_fromStart( true ),
+      m_searchRunning( false )
 {
     setObjectName( "SearchLineEdit" );
     setClearButtonShown( true );
@@ -34,6 +38,7 @@ SearchLineEdit::SearchLineEdit( QWidget * parent, Okular::Document * document )
              this, SLOT( startSearch() ) );
 
     connect(this, SIGNAL( textChanged(const QString &) ), this, SLOT( slotTextChanged(const QString &) ));
+    connect(this, SIGNAL( returnPressed(const QString &) ), this, SLOT( slotReturnPressed(const QString &) ));
     connect(document, SIGNAL( searchFinished(int, Okular::Document::SearchStatus) ), this, SLOT( searchFinished(int, Okular::Document::SearchStatus) ));
 }
 
@@ -85,10 +90,27 @@ void SearchLineEdit::setSearchFromStart( bool fromStart )
     m_fromStart = fromStart;
 }
 
+bool SearchLineEdit::isSearchRunning() const
+{
+    return m_searchRunning;
+}
+
 void SearchLineEdit::restartSearch()
 {
     m_inputDelayTimer->stop();
-    m_inputDelayTimer->start( 500 );
+    m_inputDelayTimer->start( 700 );
+    m_changed = true;
+}
+
+void SearchLineEdit::stopSearch()
+{
+    if ( m_id == -1 || !m_searchRunning )
+        return;
+
+    m_inputDelayTimer->stop();
+    // ### this should just cancel the search with id m_id, not all of them
+    m_document->cancelSearch();
+    // flagging as "changed" so the search will be reset at the next one
     m_changed = true;
 }
 
@@ -98,7 +120,11 @@ void SearchLineEdit::findNext()
         return;
 
     if ( !m_changed )
+    {
+        emit searchStarted();
+        m_searchRunning = true;
         m_document->continueSearch( m_id, m_searchType );
+    }
     else
         startSearch();
 }
@@ -109,15 +135,25 @@ void SearchLineEdit::findPrev()
         return;
 
     if ( !m_changed )
+    {
+        emit searchStarted();
+        m_searchRunning = true;
         m_document->continueSearch( m_id, m_searchType );
+    }
     else
         startSearch();
 }
 
 void SearchLineEdit::slotTextChanged( const QString & text )
+{
+    prepareLineEditForSearch();
+    restartSearch();
+}
+
+void SearchLineEdit::prepareLineEditForSearch()
 {
     QPalette pal = palette();
-    const int textLength = text.length();
+    const int textLength = text().length();
     if ( textLength > 0 && textLength < m_minLength )
     {
         const KColorScheme scheme( QPalette::Active, KColorScheme::View );
@@ -131,7 +167,13 @@ void SearchLineEdit::slotTextChanged( const QString & text )
         pal.setColor( QPalette::Text, qAppPalette.color( QPalette::Text ) );
     }
     setPalette( pal );
-    restartSearch();
+}
+
+void SearchLineEdit::slotReturnPressed( const QString &text )
+{
+    m_inputDelayTimer->stop();
+    prepareLineEditForSearch();
+    findNext();
 }
 
 void SearchLineEdit::startSearch()
@@ -148,6 +190,8 @@ void SearchLineEdit::startSearch()
     QString thistext = text();
     if ( thistext.length() >= qMax( m_minLength, 1 ) )
     {
+        emit searchStarted();
+        m_searchRunning = true;
         m_document->searchText( m_id, thistext, m_fromStart, m_caseSensitivity,
                                 m_searchType, m_moveViewport, m_color );
     }
@@ -178,6 +222,62 @@ void SearchLineEdit::searchFinished( int id, Okular::Document::SearchStatus endS
         pal.setColor( QPalette::Text, qAppPalette.color( QPalette::Text ) );
         setPalette( pal );
     }
+
+    m_searchRunning = false;
+    emit searchStopped();
+}
+
+
+SearchLineWidget::SearchLineWidget( QWidget * parent, Okular::Document * document )
+    : QWidget( parent )
+{
+    QHBoxLayout *layout = new QHBoxLayout( this );
+    layout->setMargin( 0 );
+
+    m_edit = new SearchLineEdit( this, document );
+    layout->addWidget( m_edit );
+
+    m_anim = new KPixmapSequenceWidget( this );
+    m_anim->setFixedSize( 22, 22 );
+    layout->addWidget( m_anim );
+    m_anim->hide();
+
+    m_timer = new QTimer( this );
+    m_timer->setSingleShot( true );
+    connect( m_timer, SIGNAL( timeout() ), this, SLOT( slotTimedout() ) );
+
+    connect( m_edit, SIGNAL( searchStarted() ), this, SLOT( slotSearchStarted() ) );
+    connect( m_edit, SIGNAL( searchStopped() ), this, SLOT( slotSearchStopped() ) );
+}
+
+SearchLineEdit* SearchLineWidget::lineEdit() const
+{
+    return m_edit;
+}
+
+void SearchLineWidget::slotSearchStarted()
+{
+    m_timer->start( 100 );
+}
+
+void SearchLineWidget::slotSearchStopped()
+{
+    m_timer->stop();
+    m_anim->hide();
+}
+
+void SearchLineWidget::slotTimedout()
+{
+    if ( m_anim->sequence().isEmpty() )
+    {
+        const KPixmapSequence seq( "process-working", 22 );
+        if ( seq.frameCount() > 0 )
+        {
+            m_anim->setInterval( 1000 / seq.frameCount() );
+            m_anim->setSequence( seq );
+        }
+    }
+    m_anim->show();
 }
 
 #include "searchlineedit.moc"
